@@ -34,6 +34,11 @@
 #include "greeter_canvas_item.h"
 #include "greeter_configuration.h"
 #include "greeter_canvas_text.h"
+#include "greeter_parser.h"
+
+/* Keep track of buttons so they can be set sensitive/insensitive */
+extern GtkButton *gtk_ok_button = NULL;
+extern GtkButton *gtk_start_again_button = NULL;
 
 static void
 apply_tint (GdkPixbuf *pixbuf, guint32 tint_color)
@@ -127,6 +132,15 @@ activate_button (GtkWidget *widget, gpointer data)
 		greeter_item_run_action_callback (id);
 }
 
+void 
+menubar_done (GtkMenuShell *menushell, gpointer data)
+{
+	GreeterItemInfo *entry_info = greeter_lookup_id ("user-pw-entry");
+	GtkWidget *entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
+
+	gtk_widget_grab_focus (entry);
+}
+
 static GtkWidget *
 make_menubar (void)
 {
@@ -181,6 +195,9 @@ make_menubar (void)
 				  G_CALLBACK (gtk_main_quit), NULL);
 	}
 
+	g_signal_connect (G_OBJECT(gtk_menu_item_get_submenu(
+                        gtk_container_get_children(GTK_CONTAINER(menubar))->data)), "selection-done", G_CALLBACK (menubar_done), NULL);
+
 	return menubar;
 }
 
@@ -230,7 +247,11 @@ greeter_options_handler (GreeterItemInfo *item, GtkWidget *menubar)
 static void
 greeter_item_run_button_action_callback (GtkButton *button, const char *id)
 {
+	GreeterItemInfo *entry_info = greeter_lookup_id ("user-pw-entry");
+	GtkWidget *entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
+
         greeter_item_run_action_callback (id);
+	gtk_widget_grab_focus (entry);
 }
 
 void
@@ -403,6 +424,12 @@ greeter_item_create_canvas_item (GreeterItemInfo *item)
   case GREETER_ITEM_TYPE_BUTTON:
     gtkbutton = gtk_button_new_with_mnemonic (item->data.text.orig_text);
     gtk_widget_set_name (gtkbutton, item->id);
+    if (strcmp (ve_sure_string (item->id), "ok_button") == 0) {
+       gtk_ok_button = GTK_BUTTON (gtkbutton);
+    } else if (strcmp (ve_sure_string (item->id), "cancel_button") == 0) {
+       gtk_start_again_button = GTK_BUTTON (gtkbutton);
+    }
+
     g_signal_connect (G_OBJECT (gtkbutton), "clicked",
                       G_CALLBACK (greeter_item_run_button_action_callback),
                       item->id);
@@ -448,7 +475,7 @@ greeter_item_create_canvas_item (GreeterItemInfo *item)
 				   "height", (double)rect.height,
 				   "width", (double)rect.width,
 				   NULL);
-	    
+
 	    greeter_item_register_action_callback ("options_button",
 						   (ActionFunc)greeter_options_handler,
 						   menubar);
@@ -481,22 +508,37 @@ greeter_item_create_canvas_item (GreeterItemInfo *item)
     /* Note a list type must be setup later and we will add the list store
      * to it then, depending on the type.  Likely userlist is the
      * only type we support */
-    list = gtk_tree_view_new ();
 
-    swin = gtk_scrolled_window_new (NULL, NULL);
-    gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (swin),
+    if (item->data.list.combo_type) {
+       list = gtk_combo_box_new_text ();
+
+       item->item = gnome_canvas_item_new (group,
+					   GNOME_TYPE_CANVAS_WIDGET,
+					   "widget", list,
+					   "x", x1,
+					   "y", y1,
+					   "height", (double)rect.height,
+					   "width", (double)rect.width,
+					   NULL);
+    } else {
+       list = gtk_tree_view_new ();
+
+       swin = gtk_scrolled_window_new (NULL, NULL);
+       gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (swin),
     					GTK_SHADOW_NONE);
-    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swin),
+       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swin),
     				GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-    gtk_container_add (GTK_CONTAINER (swin), list);
-    item->item = gnome_canvas_item_new (group,
-					GNOME_TYPE_CANVAS_WIDGET,
-					"widget", swin,
-					"x", x1,
-					"y", y1,
-					"height", (double)rect.height,
-					"width", (double)rect.width,
-					NULL);
+       gtk_container_add (GTK_CONTAINER (swin), list);
+
+       item->item = gnome_canvas_item_new (group,
+					   GNOME_TYPE_CANVAS_WIDGET,
+					   "widget", swin,
+					   "x", x1,
+					   "y", y1,
+					   "height", (double)rect.height,
+					   "width", (double)rect.width,
+					   NULL);
+    }
 
     break;
   }
